@@ -1,5 +1,5 @@
 /* This file is part of the Zebra server.
-   Copyright (C) 1994-2010 Index Data
+   Copyright (C) 1994-2011 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -18,6 +18,9 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 
+#if HAVE_CONFIG_H
+#include <config.h>
+#endif
 #include <assert.h>
 #include <yaz/log.h>
 
@@ -123,7 +126,7 @@ static struct recType grs_type_sgml =
 };
 
 RecType
-#ifdef IDZEBRA_STATIC_GRS_SGML
+#if IDZEBRA_STATIC_GRS_SGML
 idzebra_filter_grs_sgml
 #else
 idzebra_filter
